@@ -26,7 +26,7 @@ prelim_AM <- function(x, Assess, ncpus = 1, ...) {
     if(inherits(x, "OM")) {
       message("Generating Hist object from OM object via runMSE:")
       runHist <- runMSE(x, Hist = TRUE)
-      Data <- runHist$Data
+      if(packageVersion("DLMtool") >= 5.3) Data <- runHist@Data else Data <- runHist$Data
     } else
       if(inherits(x, "Data")) {
         Data <- x
@@ -95,7 +95,11 @@ prelim_AM <- function(x, Assess, ncpus = 1, ...) {
 #' @export
 diagnostic_AM <- function(MSE, MP = NULL, gradient_threshold = 0.1, figure = TRUE) {
   if(!inherits(MSE, "MSE")) stop("No object of class MSE was provided.")
-  if(length(MSE@Misc) == 0) stop("Nothing found in MSE@Misc. Use an MP created by 'make_MP(diagnostic = 'min')' and set 'runMSE(PPD = TRUE)'.")
+  if(packageVersion("DLMtool") >= 5.3) {
+    if(length(MSE@Misc$Data) == 0) stop("Nothing found in MSE@Misc$Data. Use an MP created by 'make_MP(diagnostic = 'min')' and set 'runMSE(PPD = TRUE)'.")
+  } else {
+    if(length(MSE@Misc) == 0) stop("Nothing found in MSE@Misc. Use an MP created by 'make_MP(diagnostic = 'min')' and set 'runMSE(PPD = TRUE)'.")
+  }
 
   if(figure) {
     old_par <- par(no.readonly = TRUE)
@@ -106,12 +110,20 @@ diagnostic_AM <- function(MSE, MP = NULL, gradient_threshold = 0.1, figure = TRU
   }
 
   MPs <- MSE@MPs
-  has_diagnostic_fn <- function(x) {
-    Misc <- x@Misc
+  has_diagnostic_fn <- function(Data) {
+    Misc <- Data@Misc
     all(vapply(Misc, function(y) any(names(y) == "diagnostic"), logical(1)))
   }
-  has_diagnostic <- vapply(MSE@Misc, has_diagnostic_fn, logical(1))
-  if(all(!has_diagnostic)) stop("No diagnostic information found in MSE@Misc for any MP. Use an MP created by 'make_MP(diagnostic = 'min')' and set 'runMSE(PPD = TRUE)'.")
+
+  if(packageVersion("DLMtool") >= 5.3) {
+    has_diagnostic <- vapply(MSE@Misc$Data, has_diagnostic_fn, logical(1))
+    if(all(!has_diagnostic)) stop("No diagnostic information found in MSE@Misc$Data for any MP. Use an MP created by 'make_MP(diagnostic = 'min')' and set 'runMSE(PPD = TRUE)'.")
+
+  } else {
+    has_diagnostic <- vapply(MSE@Misc, has_diagnostic_fn, logical(1))
+    if(all(!has_diagnostic)) stop("No diagnostic information found in MSE@Misc for any MP. Use an MP created by 'make_MP(diagnostic = 'min')' and set 'runMSE(PPD = TRUE)'.")
+  }
+
   MPs <- MPs[has_diagnostic]
 
   if(!is.null(MP)) {
@@ -125,7 +137,9 @@ diagnostic_AM <- function(MSE, MP = NULL, gradient_threshold = 0.1, figure = TRU
   get_code <- function(x, y) vapply(x, getElement, numeric(1), y)
   get_code_char <- function(x, y) vapply(x, getElement, character(1), y)
   for(i in 1:length(MPs)) {
-    objects <- MSE@Misc[[which(MPs[i] == MSE@MPs)]]
+    if(packageVersion("DLMtool") >= 5.3) {
+      objects <- MSE@Misc$Data[[which(MPs[i] == MSE@MPs)]]
+    } else objects <- MSE@Misc[[which(MPs[i] == MSE@MPs)]]
     diagnostic <- lapply(objects@Misc, getElement, "diagnostic")
 
     if(!is.null(diagnostic)) {
